//
// Author: Corey Schram
// Description: Controller for Node-MVC. This acts as the entry point of the
// application.
//

var express = require('express'),
    util = require('./utilities');

// Create the application Controller
// Takes a configuration object with the following required properties:
//   "path" - directory path the application is running in
//   "middleware" - array of connect middleware
//   "modules" - array of module names to use
exports.createController = function (config) {
    var app = express.createServer.apply(express, config.middleware),
        modulePath, routeFile, viewFile, middlewareFile,
        route, args;
        
    // Make the configuration viewable outside of this module
    // from the utilities module.
    util.config = config;

    // Go through each module and link routes
    for (var i = 0; i < config.modules.length; i++) {
        // Load module files
        modulePath = config.path + '/' + config.modules[i];
        routeFile  = require(modulePath + '/routes');
        viewFile   = require(modulePath + '/views');

        for (var j = 0; j < routeFile.routes.length; j++) {
            route = routeFile.routes[j];
            // Build arguments to the express route
            args = [route[1]];
            // Link middleware
            if (route.length > 3) {
                // Load the middleware file if it hasn't been loaded
                if (!middlewareFile) {
                    middlewareFile = require(modulePath + '/middleware');
                }
                for (var k = 2; k < (route.length - 1); k++) {
                    // If the middleware method is undefined, an error will be thrown
                    args.push(middlewareFile[route[k]]);
                }
            }
            // Link the view method. If it is undefined, an error will be thrown
            args.push(viewFile[route[route.length - 1]]);
            // Link the route
            app[route[0]].apply(app, args);
        }
        
        // Make sure the middlewareFile is cleared
        middlewareFile = null;
    }

    // Content Routing
    // Temporary, for now.
    app.get('/content/:dir/:subdir?/:file', function (req, res) {
        var file = 'content/' + req.params.dir + '/';
        if (req.params.subdir) {
            file = file + req.params.subdir + '/' + req.params.file;
        } else {
            file = file + req.params.file;
        }
        res.sendfile(file);
    });

    // Return back the ExpressJS App
    return app;
};