//
// Author: Corey Schram
// Description: Model and ModelField definitions for CouchDB.
//

var console = require('console');

var model = require('../model'),
    util = require('../utilities'),
    filter = require('../filter'),
    CouchClient = require('couch-client');
    
exports.Model = model.Model.extend({
    init : function (value, callback) {
        this._client = CouchClient(util.config.db + this._database);
        if (value) {
            if (typeof value === 'object') {
                this._super(value);
                callback && callback();
            } else {
                var _super = this._super,
                    self = this;
                this._client.get(value, function (err, doc) {
                    if (err) {
                        callback && callback(err);
                    } else {
                        _super.call(self, doc);
                        self._id = doc._id;
                        self._rev = doc._rev;
                        callback && callback();
                    }
                });
            }
        } else {
            this._super({});
        }
    },
    save : function (callback) {
        console.log('saving');
        var values = this._super(),
            self = this;
        if (this._id) {
            values._id = this._id;
        }
        this._client.save(values, function (err, doc) {
            if (err) {
                callback && callback(err);
            } else {
                self._id = doc._id;
                self._rev = doc._rev;
                callback && callback();
            }
        });
    }
});

exports.query = function (model, filters, callback) {
    var client = CouchClient(util.config.db);
    client.view('/' + model.prototype._database + '/_all_docs/', function (err, doc) {
        if (err) {
            callback(err);
        } else {
            var set = [],
                count = doc.rows.length,
                modelCallback;
            modelCallback = function () {
                count--;
                if (count === 0) {
                    callback(null, filter.filter(set, filters));
                }
            }
            for (var i = 0; i < count; i++) {
                set.push(new model(doc.rows[i].id, modelCallback));
            }
        }
    });
};