//
// Author: Corey Schram
// Description: Utilities for querying and filtering models
//

exports.filter = function (set, filters) {
    var filteredSet = [],
        exclude;
    for (var i = 0; i < set.length; i++) {
        exclude = false;
        for (var filterName in filters) {
            if (filters.hasOwnProperty(filterName)) {
                if (typeof filters[filterName] === 'function') {
                    if (!filters[filterName](set[i][filterName])) {
                        exclude = true;
                        break;
                    }
                } else {
                    if (set[i][filterName] != filters[filterName]) {
                        exclude = true;
                        break;
                    }
                }
            }
        }
        if (!exclude) {
            filteredSet.push(set[i]);
        }
    }
    return filteredSet;
};

exports.greater = function (lower) {
    return function (value) {
        return value > lower;
    };
};

exports.greaterEqual = function (lower) {
    return function (value) {
        return value >= lower;
    };
};

exports.less = function (upper) {
    return function (value) {
        return upper > value;
    };
};

exports.lessEqual = function (upper) {
    return function (value) {
        return upper >= value;
    };
};

exports.notEqual = function (exclude) {
    return function (value) {
        return value != exclude;
    };
};