//
// Author: Corey Schram
// Description: Base classes for database Model interaction.
//

var console = require('console');

var Class = require('./inheritance').Class,
    util = require('./utilities');

// Base Model Field
// Fields are the glue between a value in a model
// and the database representation.
exports.ModelField = Class.extend({
    defaultOptions : {
        required : false,
        defaultValue : 0
    },
    init : function (options) {
        this.options = options || {};
        util.extend(this.options, this.defaultOptions);
    },
    load : function (key, value) {
        return value;
    },
    save : function (key, value) {
        return value;
    }
});

// Base Model
// Container for model fields. Acts as the glue between model
// fields and the actual database.
exports.Model = Class.extend({
    init : function (values) {
        for (var fieldName in this._fields) {
            if (this._fields.hasOwnProperty(fieldName)) {
                if (values.hasOwnProperty(fieldName)) {
                    this[fieldName] = this._fields[fieldName].load(fieldName, values[fieldName]);
                } else {
                    this[fieldName] = this._fields[fieldName].options.defaultValue;
                }
            }
        }
    },
    save : function () {
        console.log('building values');
        var values = {};
        for (var fieldName in this._fields) {
            if (this._fields.hasOwnProperty(fieldName)) {
                if (this[fieldName]) {
                    values[fieldName] = this._fields[fieldName].save(fieldName, this[fieldName]);
                } else if (this._fields[fieldName].options.required) {
                    values[fieldName] = this._fields[fieldName].save(
                        fieldName,
                        this._fields[fieldName].options.defaultValue
                    );
                }
            }
        }
        console.log('values: ' + JSON.stringify(values));
        return values;
    }
});